import { useEffect, useState } from "react";
import Overlay from "../Overlay/Overlay";
import { calculateOptimalPosition } from "../../Services/OptimalPosition/CalculateOptimalPosition";
import "../Overlay/Overlay.css";
import "./Tourguide.css"; // Custom CSS file
import { useViewport } from "../../Hooks/useViewport";
import { ITourGuideProps, ColorConfig } from "./Interface.type";
import { IPosition } from "../../Services/OptimalPosition/Interface.type";
import { Button } from "../Button/Button";

// The TourGuide component provides a step-by-step guide interface for users.
export const TourGuide: React.FC<ITourGuideProps> = ({
  steps,
  colorConfig: userColorConfig = {},
}) => {
  // Get viewport width using a custom hook to determine device type
  const viewportWidth = useViewport();
  const isMobile = viewportWidth <= 600;
  const isTablet = viewportWidth > 600 && viewportWidth <= 1024;

  // State to control visibility, current step, and popup position
  const [isVisible, setIsVisible] = useState(false);
  const [currentStep, setCurrentStep] = useState(0);
  const [popupPosition, setPopupPosition] = useState<IPosition | null>(null);

  // Merge user-defined and default color configurations
  const [colorConfig] = useState<ColorConfig>({
    Modal: userColorConfig.Modal || "",
    textColor: userColorConfig.textColor || "",
    buttonColor: userColorConfig.buttonColor || "",
    overlayBackgroundColor: userColorConfig.overlayBackgroundColor || "",
    highlightColor: userColorConfig.highlightColor || "",
  });

  // Show the tour guide if steps are available
  useEffect(() => {
    if (steps.length) setIsVisible(true);
  }, [steps]);

  // Update the position of the popup based on the current step's element
  const updatePopupPosition = () => {
    const step = steps[currentStep];
    const position = calculateOptimalPosition(step.id, 200, 100, {
      x: 10,
      y: 10,
    });
    setPopupPosition(position);
  };

  // Update the popup position and scroll the current element into view
  useEffect(() => {
    updatePopupPosition();

    const element = document.getElementById(steps[currentStep]?.id);
    if (element) {
      element.scrollIntoView({ behavior: "smooth", block: "center" });
    }

    // Recalculate the popup position on scroll
    const handleScroll = () => {
      updatePopupPosition();
    };

    window.addEventListener("scroll", handleScroll);
    return () => {
      window.removeEventListener("scroll", handleScroll);
    };
  }, [currentStep, steps]);

  // Update the popup position when the window is resized
  useEffect(() => {
    const handleResize = () => {
      updatePopupPosition();
    };

    window.addEventListener("resize", handleResize);
    return () => window.removeEventListener("resize", handleResize);
  }, [currentStep, steps]);

  // Function to handle text-to-speech
  const handleSpeech = () => {
    const step = steps[currentStep];
    const speech = new SpeechSynthesisUtterance();
    speech.text = `${step.title}. ${step.description}`;
    speech.lang = "en-US"; // You can change this language as needed
    window.speechSynthesis.speak(speech);
  };

  // Keyboard Navigation
  useEffect(() => {
    const handleKeyDown = (event: KeyboardEvent) => {
      if (!isVisible) return;

      if (event.key === "ArrowRight") {
        handleNext();
      } else if (event.key === "ArrowLeft") {
        handlePrevious();
      } else if (event.key === "Escape") {
        handleClose();
      }
    };

    window.addEventListener("keydown", handleKeyDown);
    return () => {
      window.removeEventListener("keydown", handleKeyDown);
    };
  }, [isVisible, currentStep, steps]);

  // Handlers for navigation and control buttons
  const handleNext = () =>
    setCurrentStep((prev) => Math.min(prev + 1, steps.length - 1));
  const handlePrevious = () => setCurrentStep((prev) => Math.max(prev - 1, 0));
  const handleClose = () => setIsVisible(false);
  const handleSkip = () => setIsVisible(false);
  const handleRestart = () => {
    setCurrentStep(0);
    setIsVisible(true);
  };

    // Extract overlay and highlight color configurations
  const { overlayBackgroundColor, highlightColor } = colorConfig;

    // Render the tour guide interface
  return (
    <>
     <Button
        label="Restart Tour"
        onClick={handleRestart}
        className="tour-restart-button"
      />

      {isVisible && popupPosition && (
        <>
          <Overlay
            activeElementId={steps[currentStep]?.id}
            onClose={handleClose}
            colorConfig={{
              overlayBackgroundColor: overlayBackgroundColor || "",
              highlightColor: highlightColor || "",
            }}
          />

          <div
            className={`tour-overlay-modal ${isMobile ? "mobile" : ""} ${
              isTablet ? "tablet" : ""
            }`}
            style={{
              top: popupPosition?.top || "auto",
              left: popupPosition?.left || "auto",
              transformOrigin: popupPosition?.transformOrigin,
            }}
          >
            <Button
              label="&times;"
              onClick={handleClose}
              className="tour-close-button"
            />

            <h3 className="tour-title">{steps[currentStep]?.title}</h3>
            <p className="tour-description">
              {steps[currentStep]?.description}
            </p>

            {steps[currentStep]?.logoUrl && (
              <div className="tour-logo-container">
                <img
                  src={steps[currentStep]?.logoUrl}
                  alt="Logo"
                  className="tour-logo"
                />
              </div>
            )}

            {/* Navigation buttons */}
            <div className="tour-button-container">
            <Button
                label="Skip"
                onClick={handleSkip}
                className="tour-button"
              />
              <Button
                label="Previous"
                onClick={handlePrevious}
                className="tour-button"
                disabled={currentStep === 0}
              />
              <span className="tour-step-indicator">
                {currentStep + 1} / {steps.length}
              </span>
              <Button
                label="Next"
                onClick={handleNext}
                className="tour-button"
                disabled={currentStep === steps.length - 1}
              />
              <Button
                label=""
                onClick={handleSpeech}
                className="mic-button"
                icon="fa fa-microphone"
              />
            </div>
          </div>
        </>
      )}
    </>
  );
};
